<?php

namespace Modules\Cotacao\Modules\ListaPreco\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use \Setforms2\Shared\ORM\DateTime;
use Exception;
use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use \Zeedhi\Framework\DB\StoredProcedure\Param;

class Sup01200MontagemListaPrecoController extends \Teknisa\GER\Controllers\TeknisaCrud
{

    protected $crudService;
    protected $service;
    protected $connection;
    protected $newCode;
    protected $dataSourceName;
    protected $dataSourceNameToSave;

    public function __construct($crudService, $parameterBag, $connection, $environment, $newCode, $service)
    {
        $this->crudService  = $crudService;
        $this->connection   = $connection;
        $this->environment  = $environment;
        $this->newCode      = $newCode;
        $this->service      = $service;
        parent::__construct($crudService, $parameterBag);
    }

    public function confirmaSenhaComprador(DTO\Request\Filter $request, DTO\Response $response)
    {
        $this->dataSourceName = 'operador';

        try {
            $filter = $request->getFilterCriteria()->getConditions();
            $conditions = array_filter($filter, function ($row) {
                return $row['columnName'] === 'CDOPERADOR';
            }, ARRAY_FILTER_USE_BOTH);
            $operador = $this->crudService->getFilteredDataSet('operador', $conditions)->getRows();

            $passWord = array_filter($filter, function ($row) {
                return $row['columnName'] === 'PASSWORD';
            }, ARRAY_FILTER_USE_BOTH);

            if (isset(end($operador)['CDSENHAOPERWEB']) && (end($operador)['CDSENHAOPERWEB'] !== md5(end($passWord)['value']))) {
                throw new Exception("INVALID_PASSWORD");
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function validaData(DTO\Request\Filter $request, DTO\Response $response)
    {
        try {
            $filterCriteria = $request->getFilterCriteria();
            $conditions = $filterCriteria->getConditions();
            $dataset = $this->crudService->getFilteredDataSet("sup01200_validaData", $conditions)->getRows();
            $response->addDataSet(new DataSet('data', $dataset));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    private function getCentral(&$row)
    {
        try {
            $conditions = array(
                array(
                    'columnName' => 'CDFILIAL',
                    'operator' => '=',
                    "value" => $this->environment->getCdFilial()
                )
            );

            $result = $this->crudService->getFilteredDataSet("filial", $conditions)->getRows();
            if (isset($result) && count($result)) {
                $filial = end($result);
                $row['P_CDFILIAL'] = isset($filial['CDFILCENTCMP']) ? $filial['CDFILCENTCMP'] : $filial['CDFILIAL'];
            } else {
                throw new \Exception('CDFILCENTCMP_NOT_REGISTERED');
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function getNewCode(&$row)
    {
        try {
            if (!isset($row['P_NRSEQPRELIST']) || empty($row['P_NRSEQPRELIST'])) {
                $row['P_NRSEQPRELIST'] = $this->newCode->getCodeByTableName('PREVLISTA' . $row['P_CDFILIAL'], $this->environment->getNrorgTrab(), 1, 10, false);
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function getExistsPrevLista($row)
    {
        try {
            $conditions = array(
                array(
                    'columnName' => 'P_CDFILIAL',
                    'operator' => '=',
                    "value" => $row['P_CDFILIAL']
                ),
                array(
                    'columnName' => 'P_NRSEQPRELIST',
                    'operator' => '=',
                    "value" => $row['P_NRSEQPRELIST']
                )
            );

            $result = $this->crudService->getFilteredDataSet("sup01200_existsPrevlista", $conditions)->getRows();
            return isset($result) && count($result);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function getParamgeral(&$row)
    {
        try {
            $result = $this->crudService->getFilteredDataSet("paramgeral", array())->getRows();
            if (isset($result) && count($result)) {
                $paramgeral = end($result);
                $row['IDSISDOS'] = isset($paramgeral['IDSISDOS']) ? $paramgeral['IDSISDOS'] : '1';
            } else {
                throw new \Exception('PARAMGERAL_NOT_REGISTERED');
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function insMontaItmPrevListGR(&$row)
    {
        try {
            $procedure = new StoredProcedure($this->connection, 'PRICE_LIST.INS_ITMPREVLIST_IDSISDOS3');
            $procedure->addParam(new Param('P_CDFILIAL',     Param::PARAM_INPUT,  $row['P_CDFILIAL'],        Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDOPERADOR',   Param::PARAM_INPUT,  $row['P_CDOPERADOR'],      Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_TCDREGIAO',    Param::PARAM_INPUT,  $row['P_CDREGIAO'],        Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDREGIAO',     Param::PARAM_INPUT,  $row['P_CDREGIAO'],        Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTINI',        Param::PARAM_INPUT,  $row['P_DTINIVALCOL'],     Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTFIN',        Param::PARAM_INPUT,  $row['P_DTFINVALCOL'],     Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_TPRODUTO',     Param::PARAM_INPUT,  $row['P_TPRODUTO'],        Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDPRODINI',    Param::PARAM_INPUT,  $row['P_CDARVPRODINI'],    Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDPRODFIN',    Param::PARAM_INPUT,  $row['P_CDARVPRODFIN'],    Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CARGCOT',      Param::PARAM_INPUT,  $row['P_IDCOTSEMMOVLIST'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRSEQPRELIST', Param::PARAM_INPUT,  $row['P_NRSEQPRELIST'],    Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_COINCIDEP',    Param::PARAM_INPUT,  $row['P_IDCOINVALIDLIST'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_ROWSAFFECT',   Param::PARAM_OUTPUT, NULL,                      Param::PARAM_TYPE_STR, 20));
            $data = $procedure->execute();
            $row['ROWSAFFECT'] = is_array($data) && count($data) && $data['P_ROWSAFFECT'] ? $data['P_ROWSAFFECT'] : 0;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function getRegiao($row)
    {
        try {
            if (isset($row['P_CDREGIAO']) && $row['P_CDREGIAO'] === 'T') {
                $dataSourceName = 'regiao';
                $conditions = array();
            } else {
                $dataSourceName = 'regiaoInstrQuery';
                $conditions = array(
                    array(
                        'columnName' => 'P_CDREGIAO',
                        'operator' => '=',
                        "value" => $row['P_CDREGIAO']
                    )
                );
            }

            return $this->crudService->getFilteredDataSet($dataSourceName, $conditions)->getRows();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function getCountItmprevlist(&$row)
    {
        try {
            $conditions = array(
                array(
                    'columnName' => 'P_CDFILIAL',
                    'operator' => '=',
                    "value" => $row['P_CDFILIAL']
                ),
                array(
                    'columnName' => 'P_NRSEQPRELIST',
                    'operator' => '=',
                    "value" => $row['P_NRSEQPRELIST']
                )
            );

            $result = $this->crudService->getFilteredDataSet("sup01200_countItmprevlist", $conditions)->getRows();

            if (isset($result) && count($result)) {
                $row['ROWSAFFECT'] = end($result)['CONTADOR'];
            } else {
                $row['ROWSAFFECT'] = 0;
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function insMontaItmPrevList(&$row)
    {
        try {
            $regioes = $this->getRegiao($row);

            foreach ($regioes as $regiao) {
                $this->service->insertItmprevlist($row, $regiao['CDREGIAO']);
            }

            $this->service->deleteItmprevlistNotExists($row);
            $this->getCountItmprevlist($row);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function insertItmPrevList(&$row)
    {
        try {           
            $procedure = new StoredProcedure($this->connection, 'PRICE_LIST.INSERT_ITMPREVLIST');
            $procedure->addParam(new Param('P_CDFILIAL',        Param::PARAM_INPUT,  $row['P_CDFILIAL'],        Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRSEQPRELIST',    Param::PARAM_INPUT,  $row['P_NRSEQPRELIST'],    Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDOPERADOR',      Param::PARAM_INPUT,  $row['P_CDOPERADOR'],      Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_LISTCDREGIAO',    Param::PARAM_INPUT,  $row['P_CDREGIAO'],        Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTINI',           Param::PARAM_INPUT,  $row['P_DTINIVALCOL'],     Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTFIN',           Param::PARAM_INPUT,  $row['P_DTFINVALCOL'],     Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_IDCOINVALIDLIST', Param::PARAM_INPUT,  $row['P_IDCOINVALIDLIST'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDARVPRODINI',    Param::PARAM_INPUT,  $row['P_CDARVPRODINI'],    Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDARVPRODFIN',    Param::PARAM_INPUT,  $row['P_CDARVPRODFIN'],    Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_IDCOTSEMMOVLIST', Param::PARAM_INPUT,  $row['P_IDCOTSEMMOVLIST'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_ROWSAFFECT',      Param::PARAM_OUTPUT, NULL,                      Param::PARAM_TYPE_STR, 20));
            $data = $procedure->execute();
            $row['ROWSAFFECT'] = (isset($data) && count($data) && isset($data['P_ROWSAFFECT'])) ? $data['P_ROWSAFFECT'] : 0;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function montarLista(DTO\Request\Row $request, DTO\Response $response)
    {
        $row = $request->getRow();

        try {
            $this->getCentral($row);
            $this->getNewCode($row);
            $this->getParamgeral($row);

            $data = array(
                'CDFILCENTCMP' => $row['P_CDFILIAL'],
                'NRSEQPRELIST' => $row['P_NRSEQPRELIST'],
                'IDSISDOS'     => $row['IDSISDOS'],
            );

            $row['P_TPRODUTO']     = isset($row['P_CDARVPRODINI']) && !empty($row['P_CDARVPRODINI']) ? null : 'T';
            $row['P_CDARVPRODINI'] = isset($row['P_CDARVPRODINI']) ? $row['P_CDARVPRODINI'] : null;
            $row['P_CDARVPRODFIN'] = isset($row['P_CDARVPRODFIN']) ? str_pad($row['P_CDARVPRODFIN'], 20, 'z', STR_PAD_RIGHT) : null;

            if ($this->getExistsPrevLista($row)) {
                $this->service->updatePrevLista($row);
            } else {
                $this->service->insertPrevlista($row);
            }

            if ($row['IDSISDOS'] === '3') {
                $this->insMontaItmPrevListGR($row);
            } else {
                $this->insertItmPrevList($row);
            }

            $data = array_merge($data, array('ROWSAFFECT' => $row['ROWSAFFECT']));

            if ($row['ROWSAFFECT'] === 0) {
                $this->service->updateItemcole($row);
                $this->service->deletePrevlist($row);
            }

            $response->addDataSet(new DataSet('data', $data));
        } catch (\Exception $e) {
            if (isset($row['P_CDFILIAL']) && isset($row['P_NRSEQPRELIST'])) {
                $this->service->deleteItmprevlist($row);
            }
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function getFieldLista(DTO\Request\Filter $request, DTO\Response $response)
    {
        try {
            $filterCriteria = $request->getFilterCriteria();
            $filterCriteria->addCondition('CDFILIAL', $this->environment->getCdFilial());
            $dataset = $this->crudService->getFilteredDataSet("getFieldNrseqprelistByOperAndFilial", $filterCriteria->getConditions())->getRows();
            
            $response->addDataSet(new DataSet('data', $dataset));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function saveItmprevlist(DTO\Request $request, DTO\Response $response)
    {
        try {
            $this->dataSourceNameToSave = 'itmprevlist';
            parent::save($request, $response);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    private function createFilterCriteria($params)
    {
        $filterCriteria = new FilterCriteria($this->dataSourceName);

        foreach ($params as $name => $value) {
            $filterCriteria->addCondition($name, '=', $value);
        }

        return $filterCriteria;
    }

    private function getCotacoes($row, $request, &$response)
    {
        try {
            $this->dataSourceName = 'sup01200_existsCotacao';
            $params = array(
                'P_CDFILIAL'     => $row['P_CDFILIAL'],
                'P_NRSEQPRELIST' => $row['P_NRSEQPRELIST']
            );
            $filterCriteria = $this->createFilterCriteria($params);
            $newRequest = new DTO\Request\Filter(
                $filterCriteria,
                $request->getMethod(),
                $request->getRoutePath(),
                $request->getUserId()
            );
            parent::find($newRequest, $response);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getCotacaoData(DTO\Request\Row $request, DTO\Response $response)
    {
        try {
            $row = $request->getRow();
            $this->getCotacoes($row, $request, $response);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function selecCotAuto(DTO\Request\Row $request, DTO\Response $response)
    {
        try {
            $row = $request->getRow();
            $procedure = new StoredProcedure($this->connection, 'PRICE_LIST.APURA_MELHORES_PRECOS');
            $procedure->addParam(new Param('P_CDFILIAL',     Param::PARAM_INPUT, $row['P_CDFILIAL'],     Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRSEQPRELIST', Param::PARAM_INPUT, $row['P_NRSEQPRELIST'], Param::PARAM_TYPE_STR));
            $procedure->execute();

            $this->getCotacoes($row, $request, $response);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteItmprevlist(DTO\Request $request, DTO\Response $response)
    {
        try {
            $this->dataSourceName = 'itmprevlist';
            parent::delete($request, $response);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function limpaItemcoleMontagem(DTO\Request\Row $request, DTO\Response $response)
    {
        try {
            $row = $request->getRow();

            $prodIni = isset($row["P_CDARVPRODINI"]) ? $row["P_CDARVPRODINI"] : null;
            $prodFin = isset($row["P_CDARVPRODFIN"]) ? $row["P_CDARVPRODFIN"] : null;
            $tProduto = isset($row["P_CDARVPRODINI"]) && isset($row["P_CDARVPRODFIN"]) ? null : "T";
            $carGot = $row["P_IDCOTSEMMOVLIST"] === 'S' ? 'T' : null;
            $tRegiao = isset($row["P_CDREGIAO"]) && $row["P_CDREGIAO"] === 'T' ? 'T' : '';

            $procedure = new StoredProcedure($this->connection, 'LIMPA_ITEMCOLE_MONTAGEM');
            $procedure->addParam(new Param('P_CDFILIAL',     Param::PARAM_INPUT,  $row["P_CDFILIAL"],        Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTINIVALITM',  Param::PARAM_INPUT,  $row["P_DTINIVALITM"],     Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTFINVALITM',  Param::PARAM_INPUT,  $row["P_DTFINVALITM"],     Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDREGIAO',     Param::PARAM_INPUT,  $row["P_CDREGIAO"],        Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_TCDREGIAO',    Param::PARAM_INPUT,  $tRegiao,                  Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDPRODINI',    Param::PARAM_INPUT,  $prodIni,                  Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDPRODFIN',    Param::PARAM_INPUT,  $prodFin,                  Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_TPRODUTO',     Param::PARAM_INPUT,  $tProduto,                 Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CARGCOT',      Param::PARAM_INPUT,  $carGot,                   Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDOPERADOR',   Param::PARAM_INPUT,  $row["P_CDOPERADOR"],      Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_COINCIDE',     Param::PARAM_INPUT,  $row["P_IDCOINVALIDLIST"], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRSEQPRELIST', Param::PARAM_INPUT,  $row["P_NRSEQPRELIST"],    Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_PREENCHETAB',  Param::PARAM_INPUT,  $row["P_PREENCHETAB"],     Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_ROWSAFFECT',   Param::PARAM_OUTPUT, null,                      Param::PARAM_TYPE_STR, 20));
            $result = $procedure->execute();
            return $result["P_ROWSAFFECT"];
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function criarLista(DTO\Request\Row $request, DTO\Response $response)
    {
        try {
            $row = $request->getRow();

            $procedure = new StoredProcedure($this->connection, 'PRICE_LIST.CRIA_LISTA_WEB');
            $procedure->addParam(new Param('P_CDFILIAL',     Param::PARAM_INPUT,  $row['P_CDFILIAL'],     Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRSEQPRELIST', Param::PARAM_INPUT,  $row['P_NRSEQPRELIST'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_ROWSAFFECT',   Param::PARAM_OUTPUT, NULL,                   Param::PARAM_TYPE_STR, 20));
            $data = $procedure->execute();

            $response->addDataSet(new DataSet('data', $data));            
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
}
